package dao

import (
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"server/global"
)

type Project struct {
	global.GVA_MODEL
	Code               string                `json:"code" form:"code" gorm:"comment:项目编号"`
	Name               string                `json:"name" form:"name" gorm:"comment:项目名称"`
	Principal          string                `json:"principal" form:"principal" gorm:"comment:负责人"`
	Price              decimal.Decimal       `json:"price" form:"price" gorm:"type:decimal(10,2);not null"`
	Level              int                   `json:"level" form:"level" gorm:"级别"`
	Customer           string                `json:"customer" form:"customer" gorm:"相关客户"`
	Illustrate         string                `json:"illustrate" form:"illustrate" gorm:"comment:说明;type:varchar(200)"`
	State              int                   `json:"state" form:"state" gorm:"comment:状态"`
	ProjectState       ProjectState          `json:"projectState" form:"projectState" gorm:"foreignKey:State;references:id;"`
	CollectionTime     string                `json:"collectionTime" form:"collectionTime" gorm:"comment:收款时间"`
	EndTime            string                `json:"endTime" form:"endTime" gorm:"comment:结束时间"`
	Files              []ProjectFile         `json:"files" form:"files" gorm:"-"`
	WorkingHours       []ProjectWorkingHours `json:"workingHours" form:"workingHours" gorm:"-"`
	WorkingHoursTotal  int64                 `json:"workingHoursTotal" form:"workingHoursTotal" gorm:"-"`
	Collection         []Collection          `json:"collection" form:"collection" gorm:"-"`
	CollectionTotal    int64                 `json:"collectionTotal" form:"collectionTotal" gorm:"-"`
	Reimbursement      []Reimbursement       `json:"reimbursement" form:"reimbursement" gorm:"-"`
	ReimbursementTotal int64                 `json:"reimbursementTotal" form:"reimbursementTotal" gorm:"-"`
}

func (Project) TableName() string {
	return "project"
}

// TODO:项目查询

// QueryProjectList 查询项目列表
func QueryProjectList(limit, offset, state int, name, time string) (project []Project, total int64, err error) {
	// 创建db
	db := global.GVA_DB.Model(&Project{})
	// 如果有条件搜索 下方会自动创建搜索语句
	if state != 0 {
		db = db.Where("state = ?", state)
	}
	if name != "" {
		db = db.Where("name LIKE ?", "%"+name+"%")
	}
	if time != "" {
		db = db.Where("DATE_FORMAT(created_at, '%Y-%m') = ?", time)
	}
	err = db.Count(&total).Error
	if err != nil {
		return
	}
	err = db.Order("id desc").Limit(limit).Offset(offset).Preload("ProjectState").Find(&project).Error
	return project, total, err
}

// QueryProjectByCode 查询单个项目  按编号
func QueryProjectByCode(code string) (project Project, err error) {
	err = global.GVA_DB.Model(&Project{}).Where("code = ?", code).Preload("ProjectState").First(&project).Error
	return
}

// QueryAllProject 查询所以项目
func QueryAllProject() (projects []Project, err error) {
	err = global.GVA_DB.Model(&Project{}).Find(&projects).Error
	return projects, err
}

// TODO:项目新增

// CreateProject 新增项目
func (p Project) CreateProject() error {
	return global.GVA_DB.Transaction(func(tx *gorm.DB) error {
		txErr := tx.Model(&Project{}).Create(&p).Error
		if txErr != nil {
			return txErr
		}

		for _, file := range p.Files {
			var isFile ProjectFile
			tx.Model(&ProjectFile{}).Where("project_code = ? AND name = ? AND genre = ?", file.ProjectCode, file.Name, file.Genre).First(&isFile)
			if isFile.ID != 0 {
				txErr := tx.Model(&ProjectFile{}).Where("id = ?", isFile.ID).Updates(&file).Error
				if txErr != nil {
					return txErr
				}
			} else {
				txErr := tx.Model(&ProjectFile{}).Create(&file).Error
				if txErr != nil {
					return txErr
				}
			}
		}
		return txErr
	})
}

// TODO:项目修改

// UpdateProject 修改项目
func (p Project) UpdateProject() error {
	return global.GVA_DB.Model(&p).Where("id = ?", p.ID).Updates(p).Error
}

// TODO:项目删除

// DeleteProject 删除项目
func DeleteProject(code string) error {
	return global.GVA_DB.Unscoped().Where("code = ?", code).Delete(&Project{}).Error
}
